@@ -14,7 +14,7 @@ use URI::Escape;
 
 use vars qw( $VERSION );
 
-$VERSION = '0.62';
+$VERSION = '0.65';
 
 =head1 NAME
 
@@ -273,6 +273,8 @@ sub display_node {
                             metadata => $node_data{metadata}
                         );
 
+    my $node_exists = $wiki->node_exists($id);
+    my $http_status = $node_exists ? undef : '404 Not Found';
     %tt_vars = (
                    %tt_vars,
                    %metadata_vars,
@@ -285,7 +287,9 @@ sub display_node {
                    oldid         => $oldid,
                    enable_gmaps  => 1,
                    wgs84_long    => $wgs84_long,
-                   wgs84_lat     => $wgs84_lat
+                   wgs84_lat     => $wgs84_lat,
+                   empty_node    => !$node_exists,
+                   read_only     => $config->read_only,
                );
 
     # Hide from search engines if showing a specific version.
@@ -307,6 +311,7 @@ sub display_node {
                                                   id            => $id,
                                                   template      => "node.tt",
                                                   tt_vars       => \%tt_vars,
+                                                  http_status   => $http_status
                                                 );
             return $output if $return_output;
             print $output;
@@ -362,6 +367,7 @@ sub display_node {
                                                 id            => $id,
                                                 template      => "home_node.tt",
                                                 tt_vars       => \%tt_vars,
+                                                http_status   => $http_status
                                             );
         return $output if $return_output;
         print $output;
@@ -371,6 +377,7 @@ sub display_node {
                                                 id            => $id,
                                                 template      => "node.tt",
                                                 tt_vars       => \%tt_vars,
+                                                http_status   => $http_status
                                             );
         return $output if $return_output;
         print $output;
@@ -537,6 +544,7 @@ sub display_edit_form {
                     edit_type       => $edit_type,
                     moderate        => $moderate,
                     deter_robots    => 1,
+                    read_only       => $config->read_only,
     );
 
     # Override some things if we were supplied with them
@@ -609,7 +617,8 @@ sub preview_edit {
             preview_above_edit_box => $self->get_cookie(
                                                    "preview_above_edit_box" ),
             checksum               => $q->escapeHTML($checksum),
-            moderate               => $moderate
+            moderate               => $moderate,
+            read_only              => $config->read_only,
         );
         my $output = $self->process_template(
                                               id       => $node,
@@ -945,8 +954,10 @@ sub show_index {
         if ( $args{format} eq "rdf" ) {
             $template = "rdf_index.tt";
             $conf{content_type} = "application/rdf+xml";
-        }
-        elsif ( $args{format} eq "plain" ) {
+        } elsif ( $args{format} eq "json" ) {
+            $template = "json_index.tt";
+            $conf{content_type} = "text/javascript";
+        } elsif ( $args{format} eq "plain" ) {
             $template = "plain_index.tt";
             $conf{content_type} = "text/plain";
         } elsif ( $args{format} eq "map" ) {
@@ -1000,6 +1011,75 @@ sub show_index {
     print $output;
 }
 
+=item B<show_metadata>
+
+  $guide->show_metadata();
+  $guide->show_metadata(type => "category");
+  $guide->show_metadata(type => "category", format => "json");
+
+Lists all metadata types, or all metadata values of a given
+type. Useful for programatically discovering a guide.
+
+As with other methods, parameters C<return_tt_vars> and
+C<return_output> can be used to return these things instead of
+printing the output to STDOUT.
+
+=cut
+sub show_metadata {
+    my ($self, %args) = @_;
+    my $wiki = $self->wiki;
+    my $formatter = $wiki->formatter;
+
+    my @values;
+    my $type;
+    my $may_descend = 0;
+    if($args{"type"} && $args{"type"} ne "metadata_type") {
+       $type = $args{"type"};
+       @values = $wiki->store->list_metadata_by_type($args{"type"});
+    } else {
+       $may_descend = 1;
+       $type = "metadata_type";
+       @values = $wiki->store->list_metadata_names;
+    }
+
+    my %tt_vars = ( type          => $type,
+                    may_descend   => $may_descend,
+                    metadata      => \@values,
+                    num_results   => scalar @values,
+                    not_deletable => 1,
+                    deter_robots  => 1,
+                    not_editable  => 1 );
+    return %tt_vars if $args{return_tt_vars};
+
+    my $output;
+    my $content_type;
+
+    if($args{"format"}) {
+       if($args{"format"} eq "json") {
+          $content_type = "text/javascript";
+          my $json = OpenGuides::JSON->new( wiki => $wiki, 
+                                            config => $self->config );
+          $output = $json->output_as_json(
+                                 $type => \@values
+          );
+       }
+    }
+    unless($output) {
+       $output = OpenGuides::Template->output(
+                                                 wiki    => $wiki,
+                                                 config  => $self->config,
+                                                 template=>"metadata.tt",
+                                                 vars    => \%tt_vars,
+                                             );
+    }
+    return $output if $args{return_output};
+
+    if($content_type) {
+       print "Content-type: $content_type\n\n";
+    }
+    print $output;
+}
+
 =item B<list_all_versions>
 
   $guide->list_all_versions ( id => "Home Page" );
@@ -1214,7 +1294,7 @@ sub display_about {
   </description>
 
   <homepage rdf:resource="http://openguides.org/" />
-  <mailing-list rdf:resource="http://openguides.org/mm/listinfo/openguides-dev/" />
+  <mailing-list rdf:resource="http://lists.openguides.org/mailman/listinfo/openguides-dev/" />
   <mailing-list rdf:resource="http://urchin.earth.li/mailman/listinfo/openguides-commits/" />
 
   <maintainer>
@@ -1324,7 +1404,7 @@ project is available from the <a href="http://dev.openguides.org/">OpenGuides
 development site</a>.
 </p>
 <p>
-Copyright &copy;2003-2007, <a href="http://openguides.org/">The OpenGuides
+Copyright &copy;2003-2008, <a href="http://openguides.org/">The OpenGuides
 Project</a>. "OpenGuides", "[The] Open Guide To..." and "The guides made by
 you" are trademarks of The OpenGuides Project. Any uses on this site are made 
 with permission.
@@ -1434,7 +1514,7 @@ sub commit_node {
         cgi_obj  => $q
     );
 
-    if ( scalar @{$fails} ) {
+    if ( scalar @{$fails} or $config->read_only ) {
         my %vars = (
             validate_failed => $fails
         );
@@ -1445,7 +1525,8 @@ sub commit_node {
                            metadata      => \%new_metadata,
                            vars          => \%vars,
                            checksum      => CGI->escapeHTML($checksum),
-                           return_output => 1
+                           return_output => 1,
+                           read_only     => $config->read_only,
         );
 
         return $output if $return_output;
@@ -1485,7 +1566,10 @@ sub commit_node {
     # Skip this for nodes needing moderation - this occurs for them once
     #  they've been moderated
     my $needs_moderation = $wiki->node_required_moderation($node);
-    unless( $needs_moderation ) {
+    my $in_moderate_whitelist
+        = OpenGuides::Utils->in_moderate_whitelist($self->config, $new_metadata{host});
+
+    if ( $in_moderate_whitelist or not $needs_moderation ) {
         $self->_autoCreateCategoryLocale(
                                           id       => $node,
                                           metadata => \%new_metadata
@@ -1496,22 +1580,30 @@ sub commit_node {
                                      \%new_metadata );
 
     if ($written) {
-        if ( $needs_moderation and $config->send_moderation_notifications ) {
-            my $body = "The node '$node' in the OpenGuides installation\n" .
-                "'" . $config->site_name . "' requires moderation. ".
-                "Please visit\n" .
-                $config->script_url . $config->script_name .
-                "?action=show_needing_moderation\nat your convenience.\n";
-            eval {
-                OpenGuides::Utils->send_email(
-                    config        => $config,
-                    subject       => "Node requires moderation",
-                    body          => $body,
-                    admin         => 1,
-                    return_output => $return_output
+        if ( $needs_moderation ) {
+            if ( $in_moderate_whitelist ) {
+                $self->wiki->moderate_node(
+                                            name    => $node,
+                                            version => $written
                 );
-            };
-            warn $@ if $@;
+            }
+            elsif ( $config->send_moderation_notifications ) {
+                my $body = "The node '$node' in the OpenGuides installation\n" .
+                    "'" . $config->site_name . "' requires moderation. ".
+                    "Please visit\n" .
+                    $config->script_url . $config->script_name .
+                    "?action=show_needing_moderation\nat your convenience.\n";
+                eval {
+                    OpenGuides::Utils->send_email(
+                        config        => $config,
+                        subject       => "Node requires moderation",
+                        body          => $body,
+                        admin         => 1,
+                        return_output => $return_output
+                    );
+                };
+                warn $@ if $@;
+            }
         }
 
         my $output = $self->redirect_to_node($node);
@@ -2168,12 +2260,13 @@ sub display_admin_interface {
 sub process_template {
     my ($self, %args) = @_;
     my %output_conf = (
-                          wiki     => $self->wiki,
-                          config   => $self->config,
-                          node     => $args{id},
-                          template => $args{template},
-                          vars     => $args{tt_vars},
-                          cookies  => $args{cookies},
+                          wiki        => $self->wiki,
+                          config      => $self->config,
+                          node        => $args{id},
+                          template    => $args{template},
+                          vars        => $args{tt_vars},
+                          cookies     => $args{cookies},
+                          http_status => $args{http_status}
                       );
     if ( $args{content_type} ) {
         $output_conf{content_type} = $args{content_type};
@@ -2209,6 +2302,7 @@ sub get_cookie {
     return $cookie_data{$pref_name};
 }
 
+=back
 
 =head1 BUGS AND CAVEATS
 
@@ -2221,11 +2315,11 @@ L<http://dev.openguides.org/>
 
 =over 4
 
-=item * L<http://london.openguides.org/|The Open Guide to London>, the first and biggest OpenGuides site.
+=item * The Open Guide to London, at L<http://london.openguides.org/>, is the first and biggest OpenGuides site.
 
-=item * L<http://openguides.org/|The OpenGuides website>, with a list of all live OpenGuides installs.
+=item * A list of live OpenGuides installs is available at L<http://openguides.org/>.
 
-=item * L<Wiki::Toolkit>, the Wiki toolkit which does the heavy lifting for OpenGuides
+=item * L<Wiki::Toolkit>, the Wiki toolkit which does the heavy lifting for OpenGuides.
 
 =back
 
@@ -2243,7 +2337,7 @@ The OpenGuides Project (openguides-dev@lists.openguides.org)
 
 =head1 COPYRIGHT
 
-     Copyright (C) 2003-2008 The OpenGuides Project.  All Rights Reserved.
+     Copyright (C) 2003-2010 The OpenGuides Project.  All Rights Reserved.
 
 The OpenGuides distribution is free software; you can redistribute it
 and/or modify it under the same terms as Perl itself.
